@@ -34,6 +34,15 @@ sub new ($$$$$) {
 }
 
 sub dbg { HTML::WebMake::Main::dbg (@_); }
+sub dbg2 { HTML::WebMake::Main::dbg2 (@_); }
+
+# -------------------------------------------------------------------------
+
+sub set_name_sed_callback {
+  my ($self, $sedobj, $sedmethod) = @_;
+  $self->{sedobj} = $sedobj;
+  $self->{sedmethod} = $sedmethod;
+}
 
 # -------------------------------------------------------------------------
 
@@ -69,18 +78,27 @@ sub parse_metatable_csv {
       @metanames = @elems; next;
     }
 
+    $contname = $self->fixname ($contname);
+
     my $contobj = $self->{main}->{contents}->{$contname};
     if (!defined $contobj) {
       $self->{main}->fail ("<metatable>: cannot find content \${$contname}");
       next;
     }
 
+    if ($#metanames < 0) {
+      $self->{main}->fail ("<metatable>: no '.' line in file");
+      next;
+    }
+
     for ($i = 0; $i <= $#elems && $i <= $#metanames; $i++) {
       my $metaname = $metanames[$i];
       my $val = $elems[$i];
 
       $contobj->create_extra_metas_if_needed();
       $contobj->{extra_metas}->{$metaname} = $val;
+
+      dbg2 ("attaching metadata \"$metaname\"=\"$val\" to content \"$contname\"");
     }
   }
 }
@@ -115,6 +133,7 @@ sub parse_metatable_xml {
   # $text = '';
 
   foreach my $contname (keys %{$self->{targetblocks}}) {
+    $contname = $self->fixname ($contname);
     my $contobj = $self->{main}->{contents}->{$contname};
     $text = $self->{targetblocks}->{$contname};
     $self->{tagging_content} = $contobj;
@@ -160,9 +179,7 @@ sub parse_xml_block {
   my $util = $self->{main}->{util};
 
   $block =~ s/^\s+//gs;
-
-  1 while $block =~ s/<\{!--.*?--\}>//gs;       # WebMake comments.
-  1 while $block =~ s/^<!--.*?-->//gs;          # XML-style comments.
+  $block =~ s/^<!--.*?-->//gs;
 
   if ($subtags eq $TARGETS) {
     $block = $util->strip_tags ($block, "target", $self, \&tag_target, qw(id));
@@ -183,6 +200,16 @@ sub parse_xml_block {
 
 # -------------------------------------------------------------------------
 
+sub fixname {
+  my ($self, $contname) = @_;
+  if (defined $self->{sedobj}) {
+    $contname = &{$self->{sedmethod}} ($self->{sedobj}, $contname);
+  }
+  $contname;
+}
+
+# -------------------------------------------------------------------------
+
 1;
 
 # METATABLE XML FORMAT
